/*globals ProportionalLegend, $ */

var MAX_LABEL_WIDTH = 150,

// the length of the shortest possible leader line
MIN_LEADER_LINE_LENGTH = 20,

// move inner circles in upward direction by VERT_CIRCLE_OFFSET x SYMBOLS_STROKE_WIDTH
VERT_CIRCLE_OFFSET = 0.05,

// stroke width for circles and lines
SYMBOLS_STROKE_WIDTH = 2.0,

// horizontal distance between end of line and label
LABEL_DIST = 5;

function canvasWidth() {"use strict";
    // $("#svgLegend").width(); does always return 100 with Firefox if the width is set to 100%
    return $("#svgContainer").width();
}

function canvasHeight() {"use strict";
    return $("#svgContainer").height();
}

function parseBreaks(breaksStr) {"use strict";
    var i, v, breaks = [];

    //split the numbers by whitespaces
    breaksStr = breaksStr.split(/\s+/);

    // convert valid strings to numbers
    for ( i = 0; i < breaksStr.length; i += 1) {
        v = parseFloat(breaksStr[i]);
        if (!isNaN(v)) {
            breaks.push(v);
        }
    }

    //order the base values
    breaks = breaks.sort(function(a, b) {
        return b - a;
    });

    return breaks;
}

function SVG(tag) {"use strict";
    return document.createElementNS('http://www.w3.org/2000/svg', tag);
}

function createCircle(cx, cy, r, parent, strokeColor) {"use strict";
    var $svg = $(parent);
    $(SVG('circle')).attr('cx', cx).attr('cy', cy).attr('r', r).attr('fill', "black").attr('stroke', strokeColor).attr('stroke-width', SYMBOLS_STROKE_WIDTH).appendTo($svg);
}

function horizontalLine(x1, x2, y, parent, color) {"use strict";
    var $svg = $(parent);
    $(SVG('line')).attr('x1', x1).attr('y1', y).attr('x2', x2).attr('y2', y).attr('stroke', color).attr('stroke-width', SYMBOLS_STROKE_WIDTH).appendTo($svg);
}

function createText(text, x, y, parent) {"use strict";
    var $svg = $(parent);
    $(SVG('text')).attr('x', x).attr('y', y).append(text).appendTo($svg);
}

function contructLegend(legend, values, largestPossibleRadius) {"use strict";

    var maxR, r, legendGroup, labelsGroup, diameters, lineX, intersectionX, d, cy, i;

    legend.setMaximumRadius(largestPossibleRadius);
    diameters = [];
    for ( i = 0; i < values.length; i += 1) {
        diameters.push(legend.valueToSymbolHeight(values[i]));
    }

    // the radius of the largest circle
    maxR = legend.valueToSymbolHeight(values[0]) / 2;

    // end of horizontal line
    lineX = (canvasWidth() - MAX_LABEL_WIDTH - MIN_LEADER_LINE_LENGTH - LABEL_DIST) / 2 + MIN_LEADER_LINE_LENGTH;

    legendGroup = $('#legendGroup')[0];
    labelsGroup = $('#labelsGroup')[0];
    $('#labelsGroup').attr("font-size", legend.getFontSize());

    for ( i = 0; i < diameters.length; i += 1) {
        // circle radius and center
        r = diameters[i] / 2;
        cy = -r;

        // vertically move the circle in upward direction to avoid a "cone effect"
        if (i > 0) {
            cy -= (i - 1) * VERT_CIRCLE_OFFSET * SYMBOLS_STROKE_WIDTH;
        }

        // draw the horizontal line
        if (i === 0) {
            horizontalLine(0, lineX, cy - r, legendGroup, "black");
        } else {
            // intersection of horizontal line and largest circle
            d = maxR - 2 * r;
            intersectionX = Math.sqrt(maxR * maxR - d * d);
            horizontalLine(0, intersectionX, cy - r, legendGroup, "white");
            horizontalLine(intersectionX, lineX, cy - r, legendGroup, "black");
        }

        // draw the label
        createText(values[i], lineX + LABEL_DIST, cy - r + legend.getFontSize() / 2 - SYMBOLS_STROKE_WIDTH, labelsGroup);

        // draw the circle
        createCircle(0, cy, diameters[i] / 2, legendGroup, i === 0 ? "black" : "white");
    }
}

function createSVG() {"use strict";
    var svg = document.getElementById("svgLegend");

    // create legend group
    $(SVG('g')).attr("id", "legendGroup").appendTo(svg);
    // create labels group
    $(SVG('g')).attr("id", "labelsGroup").appendTo(svg);
}

function updateSVG() {"use strict";
    var legend, tx, ty, transform, svg, w, h, maxCircleDiameter, maxTypeSize, lineAndTextWidth;

    svg = document.getElementById("svgLegend");

    $("#legendGroup").empty();
    $("#labelsGroup").empty();

    // the origin of the legend is the lower central point where all circles meet
    // position this origin to center the legend on the canvas
    w = canvasWidth();
    h = canvasHeight();
    maxTypeSize = $("#typeSize").slider("option", "max");
    lineAndTextWidth = LABEL_DIST + MAX_LABEL_WIDTH + MIN_LEADER_LINE_LENGTH;
    maxCircleDiameter = Math.min(w - lineAndTextWidth, h - maxTypeSize) - SYMBOLS_STROKE_WIDTH;

    tx = (w - lineAndTextWidth) / 2;
    ty = h / 2 + maxCircleDiameter / 2;
    transform = "translate(" + tx + "," + ty + ")";
    $("#legendGroup").attr('transform', transform);
    $("#labelsGroup").attr('transform', transform);

    legend = new ProportionalLegend(w, h);
    legend.setMaximum($("#numberMaximumCounter").val());
    legend.setMinimum($("#numberMinimumCounter").val());
    legend.setSizeScalePerc($("#legendSize").slider("value"));
    legend.setFontSize($("#typeSize").slider("value"));
    legend.setBases(parseBreaks($('#breaks').val()));
    // write ordered bases to UI
    $('#breaks').val(legend.getBasesString());
    legend.setMinDistancePerc($("#distance").slider("value"));
    contructLegend(legend, legend.getValues(), maxCircleDiameter / 2);
}

function minMaxChange() {"use strict";
    var legend, minimum, maximum;
    legend = new ProportionalLegend(canvasWidth(), canvasHeight());
    minimum = $("#numberMinimumCounter").val();
    maximum = $("#numberMaximumCounter").val();
    if (isNaN(parseFloat(maximum))) {
        $("#numberMaximumCounter").val(legend.getMaximum()).change();
        return;
    }

    if (isNaN(parseFloat(minimum))) {
        $("#numberMinimumCounter").val(legend.getMinimum()).change();
        return;
    }

    if (minimum < maximum) {
        $("#numberMaximumCounter").val(maximum);
        $("#numberMinimumCounter").val(minimum);
    } else {
        $("#numberMaximumCounter").val(minimum);
        $("#numberMinimumCounter").val(maximum);
    }

    updateSVG();
}

function initLegendSizeSlider(legendSizePerc) {"use strict";
    function writeValue(nbr) {
        $("#legendSizeLabel").text(nbr + "%");
    }

    function action(event, ui) {
        $("#legendSize").slider('value', ui.value);
        writeValue(ui.value);
        updateSVG();
    }


    $("#legendSize").slider({
        orientation : "horizontal",
        range : "min",
        min : 3,
        max : 100,
        value : legendSizePerc,
        step : 1,
        slide : action
    });

    writeValue($("#legendSize").slider("value"));
}

function initTypeSizeSlider(fontSize) {"use strict";
    function writeValue(nbr) {
        $("#typeSizeLabel").text(nbr + "px");
    }

    function action(event, ui) {
        $("#typeSize").slider('value', ui.value);
        writeValue(ui.value);
        updateSVG();
    }


    $("#typeSize").slider({
        orientation : "horizontal",
        range : "min",
        min : 7,
        max : 32,
        value : fontSize,
        step : 1,
        slide : action
    });
    writeValue($("#typeSize").slider("value"));
}

function initDistanceSlider(minDistPerc) {"use strict";
    function writeValue(nbr) {
        $("#distanceSizeLabel").text(nbr + "%");
    }

    function action(event, ui) {
        $("#distance").slider('value', ui.value);
        writeValue(ui.value);
        updateSVG();
    }


    $("#distance").slider({
        orientation : "horizontal",
        range : "min",
        min : 50,
        max : 350,
        value : minDistPerc,
        step : 1,
        slide : action
    });
    writeValue($("#distance").slider("value"));
}

function initProportionalSymbolLegend() {"use strict";

    var legend = new ProportionalLegend(canvasWidth(), canvasHeight());
    initLegendSizeSlider(legend.getSizeScalePerc());
    initTypeSizeSlider(legend.getFontSize());
    initDistanceSlider(legend.getMinDistancePerc());

    $("#numberMaximumCounter").bind("change", minMaxChange);
    $("#numberMinimumCounter").bind("change", minMaxChange);
    $('#breaks').bind("change", updateSVG);
    createSVG();
    updateSVG();

    //$(window).resize(updateSVG);

    (function() {
        var resizeTimeout;
        function resizeThrottler() {
            // ignore resize events as long as an updateSVG execution is in the queue
            if (!resizeTimeout) {
                resizeTimeout = setTimeout(function() {
                    resizeTimeout = null;
                    updateSVG();
                }, 500);
            }
        }

        window.addEventListener("resize", resizeThrottler, false);
    })();

}


$(window).load(initProportionalSymbolLegend);
